; $Id: asesmp.lsp,v 1.7 1995/12/20 03:43:46 kirila Exp $
;;;
;;;    asesmp.lsp
;;;
;;;    Copyright (C) 1990, 1992, 1994, 1995, 1996 by Autodesk, Inc.
;;;
;;;    Permission to use, copy, modify, and distribute this software
;;;    for any purpose and without fee is hereby granted, provided
;;;    that the above copyright notice appears in all copies and
;;;    that both that copyright notice and the limited warranty and
;;;    restricted rights notice below appear in all supporting
;;;    documentation.
;;;
;;;    AUTODESK PROVIDES THIS PROGRAM "AS IS" AND WITH ALL FAULTS.
;;;    AUTODESK SPECIFICALLY DISCLAIMS ANY IMPLIED WARRANTY OF
;;;    MERCHANTABILITY OR FITNESS FOR A PARTICULAR USE.  AUTODESK, INC.
;;;    DOES NOT WARRANT THAT THE OPERATION OF THE PROGRAM WILL BE
;;;    UNINTERRUPTED OR ERROR FREE.
;;;
;;;    Use, duplication, or disclosure by the U.S. Government is subject to
;;;    restrictions set forth in FAR 52.227-19 (Commercial Computer
;;;    Software - Restricted Rights) and DFAR 252.227-7013(c)(1)(ii)
;;;    (Rights in Technical Data and Computer Software), as applicable.
;;;
;;; DESCRIPTION
;;;
;;;    This module contains the following functions:
;;;	   connect
;;;	   do
;;;	   lpn
;;;	   link
;;;	   lstat
;;;	   err
;;;
;;;    1. connect - Connect to environment
;;;
;;;    2. disconn - Disconnect from environment
;;;
;;;    3. do - Database Object Reference statistic
;;;		       This command demostrates how to get the
;;;		       information by the given Database Object Reference.
;;;
;;;	   Command: do
;;;	   Enter DO path:
;;;
;;;	       This function prints the information, related to specified DO:
;;;		   - DO path code
;;;		   - all of the contained names
;;;		   - status information
;;;		   - updatability
;;;		   - related Link Path Names
;;;		   - subordinate DO names
;;;		   - quantity of the related links
;;;		   - quantity of the related entities
;;;
;;;    4. lpn - Creating, Erasing, Renaming of LPN(s)
;;;
;;;	   Command: lpn
;;;	   View/Erase/Rename/Create/<eXit>:
;;;	       View    - displays Table path and key column names of LPN
;;;	       Erase   - erases LPN
;;;	       Rename  - renames LPN
;;;	       Create  - creates a new LPN
;;;	       eXit    - terminates function
;;;
;;;    5. link - Creating, Erasing, Updating of links
;;;
;;;	   Command: link
;;;	   View/Erase/Update/Create/<eXit>:
;;;	       View    - displays link
;;;	       Erase   - erases link
;;;	       Update  - updates link
;;;	       Create  - creates a new link
;;;	       Row     - display link row
;;;	       eXit    - terminates function
;;;
;;;    6. lstat - Links Statistic
;;;			This command demonstrates how to get
;;;			the link information
;;;			for the selected drawing objects.
;;;
;;;	   Command: lstat
;;;	   Select objects:
;;;
;;;	       This command asks for the drawing objects selecting and
;;;	       prints the statistic for the link information,
;;;	       related with the selected entities :
;;;		   - total links quantity
;;;		   - total links quantity per each LPN
;;;		   - Entity Links quantity per each LPN
;;;		   - DA links quantity per each LPN
;;;
;;;    7. err - prints ASE error stack
;;;
;;;	   Command: err
;;;
;;;
;;;
;;;
;;; GLOBALS:
;;; SESSION				; Current session descriptor
;;;
;;;
;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defun *error* (msg / i n)
   (terpri)
   (princ msg)
   (setq n (ase_errqty))
   (setq i 0)
   (while (< i n)
       (progn
	   (terpri)
	   (princ i)
	   (princ ": ")
	   (princ (ase_errmsg i))
	   (princ " dsc=")
	   (princ (ase_errdsc i))
	   (princ " code=")
	   (princ (ase_errcode i))
	   (setq i (+ i 1))
       )
   )
)
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defun print_status (status)
    (terpri)
    (if (null status)  (princ "Error: wrong data")
	(progn
	    (princ "Status: ")
	    (princ (if (= 0 status) "undefined " ""))
	    (princ (if (/= 0 (Boole 1 status 1)) "current " ""))
	    (princ (if (/= 0 (Boole 1 status 2)) "registered " ""))
	    (princ (if (/= 0 (Boole 1 status 4)) "accessible " ""))
	    (princ (if (/= 0 (Boole 1 status 8)) "connected " ""))
	    (princ (if (/= 0 (Boole 1 status 16)) "? " ""))
	)
    )
    (terpri)
)
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defun mssg (str)
   (terpri)
   (princ str)
   (terpri)
)
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defun set-CMDDIA ()
   (setq CMDDIA (getvar "CMDDIA"))
   (setvar "CMDDIA" 0)
)
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defun restore-CMDDIA ()
   (setvar "CMDDIA" CMDDIA)
)
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defun getdo (promptt)
   (getstring promptt)
)
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;	Test if asilisp.arx and ase.arx are loaded
;;
(defun inlist (lst str / ct tmpstr ret)
  (setq ct 0)
  (repeat (length lst)
    (setq tmpstr (nth ct lst))
    (if (wcmatch (strcase tmpstr) (strcat "*" str "*"))
      (setq ret T)
    )
    (setq ct (1+ ct))
  )
  ret
)
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;	Load asilisp.arx application
;;
(defun maybe-load-ase-asilisp ( / err)
  (if (not (inlist (arx) "ASILISP") )
    (progn
      (setq err (arxload "ASILISP" nil))
      (if (not err)
	(mssg "Error: ASILISP.ARX not loaded")))
  )
  (if (not (inlist (arx) "ASE") )
    (progn
      (setq err (arxload "ASE" nil))
      (if (not err)
	(mssg "Error: ASE.ARX not loaded")))
  )
)
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defun getlpn (promptt / lpn lst)
   (setq lpn "?")
   (while (equal lpn "?")
       (progn
	   (setq lpn (getstring promptt))
	   (if (equal lpn "?")
	       (if (setq lst (ase_lplist))
		   (princ lst)
		   (progn
		       (*error* "Empty list")
		       (setq lpn nil)
		       (setq lpn lpn)
		   )
	       )
	   )
       )
   )
   (if (equal lpn "")
       (setq lpn nil)
       (setq lpn lpn)
   )
)
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defun getlink (promptt linksel / link)
   (setq cnt 0)
   (if linksel
      (repeat (fix (ase_lsqty linksel) )
	  (setq n (ase_lsget linksel cnt))
	  (setq promptt (strcat promptt "\n" (itoa (fix n))))
	  (setq cnt (1+ cnt))
      )
      (*error* "Empty list")
   )

   (setq link (getstring (strcat promptt "\n")))
   (atof link)
)


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;	Return Table Name from PATH
;;
(defun get-tabname (str / ct len)
  (setq len (strlen str))
  (setq ct len)
  (while (/= "." (substr str ct 1))
    (setq ct (1- ct))
  )
  (substr str (1+ ct) (- len ct))
)
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defun getkey (path / lst col header-list)
   (if (null SESSION)
     (*error* "Environment is not connected")
     (progn
       (setq CURSOR (asi_prepare SESSION (strcat "SELECT * FROM " (get-tabname path))))
       (setq header-list (asi_coldsc CURSOR))	; header of table
       (asi_deallocstm CURSOR)
       (setq str "")
       (foreach x header-list (setq str (strcat str (cdar x) "\n" )) )
       (setq str (strcat "\nEnter key column name:\n" str ))
       (setq lst nil)
       (setq col "")
       (while col
	  (progn
	      (setq col (getstring str ))
	      (if (equal col "")
		  (setq col nil)
		  (setq lst (cons col lst))
	      )
	  )
       )
       (setq lst (reverse lst))
     ) )
)
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defun getlinkid ()
   (initget 4)
   (getreal "\nEnter ID of existing link: ")
)
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defun printlink (link / i flag item)
   (setq i 0)
   (while (setq flag (nth i link))
       (progn
	   (setq i (+ i 1))
	   (if (setq item (nth i link))
	       (progn
		   (terpri)
		   (if (= flag 1)
		       (progn
			   (princ "ID: ")
			   (princ item)
		       )
		   )
		   (if (= flag 2)
		       (progn
			   (princ "Type: ")
			   (if (= item 2)
			       (princ "DA")
			       (princ "Entity")
			   )
		       )
		   )
		   (if (= flag 3)
		       (progn
			   (princ "Status: ")
			   (princ item)
		       )
		   )
		   (if (= flag 4)
		       (progn
			   (princ "Entity name: ")
			   (princ item)
		       )
		   )
		   (if (= flag 5)
		       (progn
			   (princ "LPN: ")
			   (princ item)
		       )
		   )
		   (if (= flag 6)
		       (progn
			   (princ "Key values: ")
			   (print item)
		       )
		   )
		   (if (= flag 7)
		       (progn
			   (princ "DA column names: ")
			   (princ item)
		       )
		   )
		   (if (= flag 9)
		       (progn
			   (princ "DA column values: ")
			   (print item)
		       )
		   )
		   (if (= flag 10)
		       (progn
			   (princ "Xref/Block: ")
			   (princ item)
		       )
		   )
		   (if (= flag 11)
		       (progn
			   (princ "Reserved attribute: ")
			   (princ item)
		       )
		   )
		   (setq i (+ i 1))
	       )
	   )
       )
   )
)
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defun getentity (/ lst ent )
   (setq lst (entsel "\nSelect object: "))
   (if lst
       (setq ent (car lst))
       (setq ent nil)
   )
)
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defun getkeyval (lpn / condd kword lst val key-attrs attrqty)
   (terpri)
   (setq key-attrs (ase_lpkey lpn))
   (setq attrqty (length key-attrs))
   (setq lst nil)
   (if (= attrqty 0)
       (princ "Wrong Link Path Name")
   )
   (if (/= attrqty 0)
      (progn
	 (princ "Enter key column values")
	 (setq condd t)
	 (while condd
	     (progn
		 (foreach n key-attrs (mssg n))
		 (setq val nil)
		 (initget 0 "Real Integer String eXit")
		 (setq kword (getkword "\nReal/Integer/String/<eXit>: "))
		 (if (not kword)
		     (setq condd nil)
		 )
		 (if (equal kword "eXit")
		     (setq condd nil)
		 )
		 (if (equal kword "Real")
		     (setq val (getreal "\nEnter real: "))
		 )
		 (if (equal kword "Integer")
		     (setq val (getint "\nEnter integer: "))
		 )
		 (if (equal kword "String")
		     (setq val (getstring "\nEnter string: "))
		 )

		 (if val
		     (progn
			 (setq attrqty (1- attrqty))
			 (setq lst (cons val lst))
		     )
		 )

		 (if (= attrqty 0)
		      (setq condd nil)
		 )
	     )
	 )
	 (setq lst (reverse lst))
      )
   )
)
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; This function prints number of links in the link selection and returns
;; this number
;;
(defun print_lsqty ( lsel / qty )

   ; initial set
   (setq qty 0.)

   (if (or (eq lsel nil)			;; bad parameter
       (eq 0 (setq qty (ase_lsqty lsel))))	;; actual number of links

       (princ "\nNo linked entity selected.")
       (princ (strcat "\nTotal links: " (rtos qty)))
   )

   ; return value
   (setq qty qty)
)
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; This function filters links to the LPN 'lpn' from link selection 'lsel'
;; and prints information of each type of links among them
;;
(defun print_lpn_ls_info (lpn lsel / ls)

   ;; make copy of the current link selection
   (if (setq ls (ase_lscopy lsel))

       (progn

	   ;; filter links to the specified lpn
	   (ase_lsintersectfilter ls 5 lpn)

	   ;; print total links info
	   (print_lpn_ls_type_info lpn ls 0)

	   ;; print entity links info
	   (print_lpn_ls_type_info lpn ls 1)

	   ;; print DA links info
	   (print_lpn_ls_type_info lpn ls 2)

	   ;; remove copy
	   (ase_lsfree ls)

       )

       ;; error
       (*error* "Can't copy link selection")
   )
)
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; This function prints information about links of type 'type'
;; among link selection 'lsel', associated with the LPN 'lpn'
;;
(defun print_lpn_ls_type_info (lpn lsel ttype / ls qty ss)

   ;; make copy of the current link selection
   (if (setq ls (ase_lscopy lsel))

       (progn

	   ;; filter ls by link type
	   (ase_lsintersectfilter ls 2 ttype)

	   ;; print message
	   (cond
	       ((= ttype 1) ; entity link
		   (princ (strcat "\nEntity Links Statistic for DOR [" lpn "]:"))
	       )
	       ((= ttype 2) ; DA link
		   (princ (strcat "\nDA Links Statistic for DOR [" lpn "]:"))
	       )
	       ((= ttype 0) ; All links
		   (princ (strcat "\nLink Statistic for DOR [" lpn "]:"))
	       )
	   )

	   ;;  print number of links of the specified type
	   (if (= 0. (setq qty (ase_lsqty ls)))
	       (princ "\nNo Links of the specified type")
	       (princ (strcat "\nLinks #" (rtos qty)))
	   )

	   ;; get selection set, associated with ls and print its length
	   (if (setq ss (ase_lsentsel ls))
	       (princ (strcat "\nEntities #" (rtos (sslength ss))))
	       (princ "\nCan't get the entity selection for link selection")
	   )

	   ;; free ls
	   (ase_lsfree ls)
       )

       ;; error
       (*error* "Can't copy link selection")
   )
)
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; This function generate new cursor name
;;
;;
(defun curs-name ()
  (if (/= (type CURS-NAME-NUMBER) 'INT)         ; init if was absent
    (setq CURS-NAME-NUMBER 0))
  (setq CURS-NAME-NUMBER (1+ CURS-NAME-NUMBER))
  (strcat "CURS-NAME" (itoa CURS-NAME-NUMBER))
)
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; This function select link attribute from
;; link attribute list
;;
(defun get-link-attr (attr-list num / fl res)
   (setq fl 0
	 res nil)
   (foreach x attr-list
      (if (= fl 0)
	 (if (= x num)
	    (setq fl 2)
	    (setq fl 1) )
	 (progn
	    (if (= fl 2)
	       (setq res x) )
	    (setq fl 0) )
      )
   )
   res
)
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; This function construct statement part KEY=VALUE AND ...
;; for SQL statement
;;
(defun add-key-values (stm attr-list keyval-list) ; / a b)
  (setq a (mapcar '(lambda (x y)
		      (if (not (numberp y))
			 (setq y (strcat "'" y "'"))
			 (setq y (rtos y))
		       )
		       (strcat x "=" y)
		   )
	     attr-list keyval-list)
  )
  (setq b (car a))
  (while (> (length a) 1)
     (setq b (strcat b " AND " (cadr a)))
     (setq a (cdr a))
  )
  (strcat stm " WHERE " b)
)
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; This function construct SELECT SQL statement
;;
;;
(defun print-row (link-id lpn / key-attrs key-values path stm)	; print row
   (if (null SESSION)
       (*error* "Environment is not connected")
       (progn
	   (setq link-info (ase_linkget link-id))

	   (setq key-attrs (ase_lpkey lpn))
	   (setq key-values (get-link-attr link-info 6))
	   (setq path (ase_lppath lpn))

	   (setq stm (strcat "SELECT * FROM " (get-tabname path)))
	   (setq stm (add-key-values stm key-attrs key-values))

	   (mssg "\"SELECT\" statement: ")
	   (mssg stm)
	   (setq STMDSC (asi_prepare SESSION stm))
	   (setq CURSOR (asi_alloc STMDSC (curs-name)))

	   (if (asi_open CURSOR)
	     (progn
		(setq row (asi_fetch CURSOR))
		(if row
		   (foreach x row (print x))
		   (mssg "Linked row is not found")
		)
	     )
	   (print "CURSOR not Opened")
	   )

	   (asi_close CURSOR)
	   (asi_delete CURSOR)
	   (asi_deallocstm STMDSC)
	)
    )
)
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;	Connection report
;;
(defun report-connect (session / conmsglist)
  (mssg (strcat "DBMS:" "\t\t\t" (asi_dbms session)))
  (mssg (strcat "SQL Object Id:" "\t" (asi_sql session)))
  (mssg (strcat "Message:" "\t\t" (asi_msg session)))
)
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;	Returns True if there is any connected environment
;;
(defun is-connected ()
   (if (or (and (not (null SESSION)) (asi_objp SESSION)) (ase_dolist 8))
       t
   )
)
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; This function allow user to select environment
;;
;;
(defun get-envs (envnames / l retval uname passw)
   (setq l "" m "")
   (foreach i envnames (setq l (strcat l " " i))
		       (setq m (strcat m " " i)) )

   (setq m (strcat "\n" m " :"))

   (setq kword (getstring m))

   (if (not (eq kword ""))
      (progn
	 (setq uname (getstring "User name: "))
	  (setq passw (getstring "Password: "))
	(list kword uname passw)
      )
      kword
   )
)
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;	This defun makes connection with environment selected by user from
;;
;;
(defun do-connect (status envname username passw)
  ;; connection with ASE and ASI environments
  (mssg "Please wait ...")
  (if (not (setq SESSION (asi_connect envname username passw)))
     (mssg "ASI environment is not connected")
     (progn
	(cond
	   ((= status 4) (command "_aseadmin" "_e" envname "_o" "_c" username passw "_x" "_x"))
	   ((= status 5) (command "_aseadmin" "_o" "_c" username passw "_x" "_x"))
	   ((= status 7) (command "_aseadmin" "_o" "_c" username passw "_x" "_x"))
	)
	(report-connect SESSION)
     )
  )
)
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;	This defun makes connection with environment selected by user from
;; for ASI only
;;
(defun do-asi-connect (envname username passw)
  ;; connection with ASI environments
  (mssg "Please wait ...")
  (if (not (setq SESSION (asi_connect envname username passw)))
     (mssg "ASI environment is not connected")
     (report-connect SESSION)
  )
)
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; connect - Connection to Environment
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defun c:connect ( / envnames retval status)
	;; make list of environments, described in asi.ini
   (set-CMDDIA)
   (maybe-load-ase-asilisp)
   (setq envnames (mapcar 'car (asi_getcfg "ENVIRONMENTS")))
   (if (listp (setq retval (get-envs envnames)))
      (progn
	 (setq status (ase_dostatus (car retval)))
	 (cond
	    ((= status 4) ; accessible, not current, not registered
		(do-connect status (car retval) (cadr retval) (caddr retval)))
	    ((= status 5) ; accessible, current, not registered
		(do-connect status (car retval) (cadr retval) (caddr retval)))
	    ((= status 7) ; accessible, current, registered
		(do-connect status (car retval) (cadr retval) (caddr retval)))
	    ((= status 13) ; connected, accessible, current, not registered
		(if (null SESSION)
		   (do-asi-connect (car retval) (cadr retval) (caddr retval)))
		(mssg "environment is connected"))
	    ((= status 14) ; connected, accessible, not current, registered
		(if (null SESSION)
		   (do-asi-connect (car retval) (cadr retval) (caddr retval)))
		(mssg "environment is connected"))
	    ((= status 15) ; connected, accessible, current, registered
		(if (null SESSION)
		   (do-asi-connect (car retval) (cadr retval) (caddr retval)))
		(mssg "environment is connected"))
	    (t	(princ (strcat "status: " (itoa status))) )
	      ;; if not connected - message in Report field of Choice dialog box: Press ASIDIAG button
	    (mssg "No environments selected")
	 )
      )
      (mssg "No environments selected")
   )
   (restore-CMDDIA)
)
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; Disconnection from all connected environments
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defun c:disconn ()
  (set-CMDDIA)
  (maybe-load-ase-asilisp)
  (if (is-connected)
     (progn
	(asi_disconnect SESSION)		    ; ASI disconnect
	(setq SESSION nil)
	(command "_aseadmin" "_o" "_d" "_x" "_x")   ; ASE disconnect
     )
     (mssg "Environment is not connected")
  )
  (restore-CMDDIA)
)
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; do - Database Object Reference statistic
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defun c:do ( / do_name subordinate_do ss ls dolist name do_name_list name_code level status i lpn lss)
   (maybe-load-ase-asilisp)
   (if (setq do_name (getdo "\nEnter DO path: "))
       (progn

	   (setq do_name_list (list "Unknown" "Environment" "Catalog" "Schema" "Table" "LPN" "Full Path" "DO Path" "SQL Table Path"))

	   (if (not (setq level (ase_dopathcode do_name)))
	       (progn
		   (*error* "Wrong DO name")
		   (exit)
	       )
	   )

	   (terpri)
	   (setq status (ase_dostatus do_name))
	   (princ (nth level do_name_list))
	   (princ "- ")
	   (princ do_name)
	   (print_status status)

	   (if (setq lpn (ase_dopathname do_name 5))
	       (progn
		   (terpri)
		   (princ "LPN '")
		   (princ lpn)
		   (princ "': ")
		   (princ (if (ase_lpisupdatable lpn) "is updatable" "isn't updatable"))
		   (terpri)
	       )
	   )

	   (if (< level 4)
	       (progn
		   (setq dolist (ase_dolist do_name))
		   (princ "Subordinate DO objects (")
		   (princ (nth (1+ level) do_name_list))
		   (princ "):")
		   (terpri)
		   (setq i 0)
		   (while (setq subordinate_do (nth i dolist))
		       (progn
			   (princ (ase_dopathname subordinate_do (+ 1 level)))
			   (terpri)
			   (setq i (1+ i))
		       )
		   )
	       )
	   )
	   (if (not (null status))
	       (if (/= 0 (Boole 1 status 2))
		   (progn
		       (setq ls (ase_lscreate -3 do_name))
		       (terpri)
		       (princ "Number of the links: ")
		       (setq lss (ase_lsqty ls))
		       (if (not (null lss))
			   (princ lss)
		       )
		       (princ "0")
		       (terpri)
		       (setq ss (ase_lsentsel ls))
		       (princ "Number of the linked objects: ")
		       (if (not (null ss))
			   (princ (sslength ss))
		       )
		       (princ "0")
		       (ase_lsfree ls)
		   )
	       )
	   )

;	   (setq i 0)
;	   (while (and dolist (setq subordinate_do (nth i dolist)))
;	       (progn
;		   (asesmpdo subordinate_do)
;		   (setq i (1+ i))
;	       )
;	   )
       )
   )

   (terpri)
)
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; lpn - Creating, Erasing, Renaming of LPN(s)
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defun c:lpn (/ condd kword lpn lpn1 path lst)

   (maybe-load-ase-asilisp)
   (setq condd t)

;  Main loop
   (while condd

       (progn
;	   Get user's key word
	   (initget 0 "View Erase Rename Create eXit")
	   (setq kword (getkword "\nView/Erase/Rename/Create/<eXit>: "))

	   (if (not kword)
;	       Exit
	       (setq condd nil)
	   )

	   (if (equal kword "eXit")
;	       Exit
	       (setq condd nil)
	   )

	   (if (equal kword "View")
;	       View LPN
	       (if (setq lpn (getlpn "\nEnter existing LPN or ? for list: "))
		   (progn
;		       Get DO path for LPN
		       (if (not (setq path (ase_lppath lpn)))
			   (*error* "Can't get DO path")
			   (progn
			       (terpri)
			       (princ "DO path: ")
			       (princ path)
			   )
		       )
;		       Get key column names for LPN
		       (if (not (setq lst (ase_lpkey lpn)))
			   (*error* "Can't get key column names")
			   (progn
			       (terpri)
			       (princ "Key column names: ")
			       (princ lst)
			   )
		       )
		   )
	       )
	   )

	   (if (equal kword "Erase")
;	       Erase LPN
	       (if (setq lpn (getlpn "\nEnter existing LPN or ? for list: "))
		   (if (ase_lperase lpn)
		       (progn
			   (terpri)
			   (princ "OK")
		       )
		       (*error* "Can't erase LPN")
		   )
	       )
	   )

	   (if (equal kword "Rename")
;	       Rename LPN
	       (if (setq lpn (getlpn "\nEnter existing LPN or ? for list: "))
		   (if (setq lpn1 (getlpn "\nEnter new LPN or ? for list: "))
		       (if (ase_lprename lpn lpn1)
			   (progn
			       (terpri)
			       (princ "OK")
			   )
			   (*error* "Can't rename LPN")
		       )
		   )
	       )
	   )

	   (if (equal kword "Create")
;	       Create new LPN
	       (if (setq path (getdo "\nEnter table path: "))
		   (if (setq lpn (getlpn "\nEnter new LPN or ? for list: "))
		       (if (setq lst (getkey path))
			   (if (ase_lpcreate path lpn lst)
			       (progn
				   (terpri)
				   (princ "OK")
			       )
			       (*error* "Can't create LPN")
			   )
		       )
		   )
	       )
	   )
       )
   )
   (terpri)
)

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; link - Creating, Erasing, Updating of links
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defun c:link (/ condd kword id lst ent lpn lst)

   (maybe-load-ase-asilisp)
   (setq condd t)

;  Main loop
   (while condd

       (progn
;	   Get user's key word
	   (initget 0 "View Erase Update Create Row eXit")
	   (setq kword (getkword "\nView/Erase/Update/Create/Row/<eXit>: "))

	   (if (not kword)
;	       Exit
	       (setq condd nil)
	   )

	   (if (equal kword "eXit")
;	       Exit
	       (setq condd nil)
	   )

	   (if (equal kword "View")
;	       View link
	       (if (setq id (getlinkid))
		   (if (setq lst (ase_linkget id))
		       (printlink lst)
		       (*error* "Can't get link")
		   )
	       )
	   )

	   (if (equal kword "Erase")
;	       Erase link
	       (if (setq id (getlinkid))
		   (if (ase_linkremove id)
		       (princ "OK")
		       (*error* "Can't erase link")
		   )
	       )
	   )

	   (if (equal kword "Update")
;	       Update link
	       (if (setq id (getlinkid))
		   (progn

		       (initget 0 "Entity Lpn Key")
		       (setq kword (getkword "\nEntity/Lpn/Key: "))

;		       Change entity
		       (if (equal kword "Entity")
			   (if (setq ent (getentity))
			       (if (ase_linkupdate 1 id 4 ent)
				   (princ "OK")
				   (*error* "Can't update link")
			       )
			   )
		       )

;		       Change LPN and Key Values
		       (if (equal kword "Lpn")
			   (if (setq lpn (getlpn "\nEnter existing LPN or ? for list: "))
			       (if (setq lst (getkeyval lpn))
				   (if (ase_linkupdate 1 id 5 lpn 6 lst)
				       (princ "OK")
				       (*error* "Can't update link")
				   )
			       )
			   )
		       )

;		       Change Key Values
		       (if (equal kword "Key")
			   (if (setq lst (getkeyval lpn))
			       (if (ase_linkupdate 1 id 6 lst)
				   (princ "OK")
				   (*error* "Can't update link")
			       )
			   )
		       )
		   )
	       )
	   )

	   (if (equal kword "Create")
;	       Create new link
;	       This sample can't create DA (only Entity link)
	       (if (setq ent (getentity))
		   (if (setq lpn (getlpn "\nEnter existing LPN or ? for list: "))
		       (if (setq lst (getkeyval lpn))
			   (if (setq id (ase_linkcreate 2 1 4 ent 5 lpn 6 lst))
			       (progn
				   (princ "ID of new link: ")
				   (princ id)
;				   (print-row id lpn)
			       )
			       (*error* "Can't create link")
			   )
		       )
		   )
	       )
	   )

	   (if (equal kword "Row")
;	       Get row of specified link
	       (if (setq ent (getentity))
		   (if (setq id (getlink "\nSelect existing Link ID from list: " (ase_lscreate 4 ent)))
		      (progn
			(setq link-info (ase_linkget id))
			(setq lpn (get-link-attr link-info 5))
			(print-row id lpn)
		      )
		      (*error* "Incorrect Link number")
		   )
		   (*error* "No link for this entity")
	       )
	   )

       )
   )
   (terpri)
)
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; lstat - Links Statistic
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defun c:lstat ( / ss do lsel lpn_list lpn i)

   (maybe-load-ase-asilisp)
   ;;  select objects
   (if (setq ss (ssget))
       (progn

	   ;; create link selection
	   (setq lsel (ase_lscreate -2 ss))

	   ;; print total number of links in the link selection
	   (if (< 0. (print_lsqty lsel))
	       (progn

		   // get the list of lpn, associated with link selection
		   (if (setq lpn_list (ase_lslpnames lsel))
		       (progn
			   (setq i 0)
			   (while (setq lpn (nth i lpn_list))
			       (progn
				   (print_lpn_ls_info lpn lsel)
				   (princ "\n")
				   (setq i (+ 1 i))
			       )
			   )
		       )

		       ;; error
		       (*error* "Can't get list of LPNs, associated with link selection")
		   )

		   ;; free link selection
		   (ase_lsfree lsel)
	       )
	   )
       )
   )

   (terpri)
)
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; err - prints ASE error stack
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defun c:err ()
   (*error* "ASE error stack:")
)
