;;;
; $Id: asismp.lsp,v 1.6 1995/12/20 03:49:50 kirila Exp $
;;;
;;;    asismp.lsp
;;;    
;;;    Copyright (C) 1995, 1996 by Autodesk, Inc.
;;;    
;;;    Permission to use, copy, modify, and distribute this software 
;;;    for any purpose and without fee is hereby granted, provided
;;;    that the above copyright notice appears in all copies and 
;;;    that both that copyright notice and the limited warranty and 
;;;    restricted rights notice below appear in all supporting 
;;;    documentation.
;;;    
;;;    AUTODESK PROVIDES THIS PROGRAM "AS IS" AND WITH ALL FAULTS.  
;;;    AUTODESK SPECIFICALLY DISCLAIMS ANY IMPLIED WARRANTY OF 
;;;    MERCHANTABILITY OR FITNESS FOR A PARTICULAR USE.  AUTODESK, INC. 
;;;    DOES NOT WARRANT THAT THE OPERATION OF THE PROGRAM WILL BE 
;;;    UNINTERRUPTED OR ERROR FREE.
;;;    
;;;    Use, duplication, or disclosure by the U.S. Government is subject to 
;;;    restrictions set forth in FAR 52.227-19 (Commercial Computer 
;;;    Software - Restricted Rights) and DFAR 252.227-7013(c)(1)(ii) 
;;;    (Rights in Technical Data and Computer Software), as applicable. 
;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;	The main ASI LISP Sample program
;;;
;;;	Samples include following .lsp programs:
;;;
;;;	ASISMP.LSP	-	this file (the main for asi samples)
;;;	CHOICDLG.LSP	-	support for selection of topic dialog
;;;	ENVSDLG.LSP	-	support for selection and connection with environment 
;;;	USERNDLG.LSP	-	dialog for user name and password 
;;;	CONNDLG.LSP	-	Connection Report message
;;;	SQLEDDLG.LSP	-	SQL Edit dialog support
;;;	CURSDLG.LSP	-	SQL Edit dialog support
;;;	MSGDLG.LSP	-	Report message dialog support
;;;	ERRMSGDLG.LSP	-	SQL Report message dialog
;;;	PARLIST.LSP	-	support for Parameter list dialog box 
;;;
;;;	ASISMP.DCL	-	.dcl (dialog box file)
;;;
;;;

;; GLOBALS:

(setq WORK-DIR "d:/ast/asilisp/regress/")	; Default path for .lsp and .dcl files
(setq DCL-FILE "asismp.dcl")		; Name of dialog file
;; DCL-ID				; Dialog ID
;; SESSION				; Current session descriptor
;; STATEMENT				; Current statement descriptor
;; CURSOR				; Current cursor descriptor
;; CURS-NAME-NUMBER			; Number, used to generate cursor name
;; UPDATELIST				; Current cursor description list

;; MESSAGES:

(setq MSG_ARX_NOT_LOADED		"\nError: ASILISP.ARX not loaded"
      MSG_ERRCONN_PRESS_ASIDIAG_BUTTON  "Error connection, press Diagn... button"
      MSG_DISCONN_ENV_FIRST		"Disconnect environment first"
      MSG_END_OF_SESSION		"End of session; DBMS: "
      MSG_ENV_IS_NOT_CONNECTED		"Environment is not connected"
      MSG_NO_ENV_SELECTED		"No environments selected"
      MSG_CONNECT_ENV_FIRST		"Connect environment first"
      MSG_TYPE_SQLSTM_IS		"The type of SQL statement is "
      MSG_SQLSTM_IS_PREPARED		"The SQL statement is prepared"
      MSG_ERROR_PREPARING		"Error preparing: press Diagn... button"
      MSG_SQLSTM_ISEMPTY		"The SQL statement is EMPTY"
      MSG_SQLSTM_ISDEALLOC		"The SQL statement is deallocated"
      MSG_STM_ISEMPTY			"The statement is EMPTY"
      MSG_STM_ISEXECUTED		"The statement is executed"
      MSG_ERROR_DEALLOC			"Error dealloc statement: press Diagn... button"
      MSG_NUMBER_ROWS_AFFECTED		"Number of the rows affected: "
      MSG_SQLSTM_IS_EXECUTED		"The SQL statement is executed"
      MSG_ERROR_IEXECUTE		"Error iexecute: press Diagn... button"
      MSG_ERROR_NEXECUTE		"Error nexecute: press Diagn... button"
      MSG_ERROR_EXECUTE			"Error execute: press Diagn... button"
      MSG_SQLSTM_IS_NOTPREPARED		"The SQL statement is not prepared"
      MSG_NO_ENVS_FOUND			"No environment found"
      MSG_ENVS_FOUND			" environments found"
      MSG_QUEST_UPD_TABLE		"Update table?"
      MSG_UPD_TABLE			"The Table is updated"
      MSG_ERR_UPD_TABLE			"Error update Table"
      MSG_ERR_NOTUPD_TABLE		"The Table is not updated"
      MSG_TABL_BOTTOM			"Bottom of the table ..."
      MSG_TABL_TOP			"Top of the table ..."
      MSG_UPDATE_ROW			"The positioned row is updated"
      MSG_QUEST_DELETE_ROW		"Delete positioned row?"
      MSG_DELETE_ROW			"The positioned row is deleted"
      MSG_DELETE_NOT_EXECUTED		"Delete command is not executed"
      MSG_ERR_DELETE			"Error: Delete command is not executed"
      MSG_CLOSE_CURS_FIRST		"Close cursor first (Press Dealloc button)"
      MSG_CANT_OPEN_CURS		"Cannot open cursor, press Diagn... button"
      MSG_QUEST_DEALLOC_STM		"Deallocate statement first"
      MSG_NUMBER_ROWS_AFFECTED		"; the number of rows affected: "
      MSG_PLEASE_WAIT			"Please wait ..."
)

;; .dcl file

;;
;;	Test if asilisp.arx is loaded 
;;
(defun inlist (lst str / ct tmpstr ret)
  (setq ct 0)
  (repeat (length lst)
    (setq tmpstr (nth ct lst))
    (if (wcmatch tmpstr (strcat "*" str "*"))
      (setq ret T)
    )
    (setq ct (1+ ct))
  )
  ret
)
;;
;;	Load asilisp.arx application
;;
(defun maybe-load-asilisp ( / err)
  (if (not (inlist (arx) "ASILISP") )
    (progn
      (setq err (arxload "ASILISP" nil))
      (if (not err) 
        (princ MSG_ARX_NOT_LOADED)))
  )
)

(defun load-onfailure (fname / lspfile)	
  (while (null lspfile)
    (setq lspfile 
      (getfiled (strcat "The .lsp file is not on the Working Directory. Select " fname " file:") WORK-DIR "lsp" 10))
  )
  (load lspfile)
)
;;	This defun is common for make dialog boxes and the lisp-program, assciated with it
;;
;;	fname		- dialog program file	
;;	dial-name  	- dialog program name 
;;	dial-prm 	- parameter
;;
(defun make-dial (fname dial-name dial-prm-list)
  (if (not (load (strcat WORK-DIR fname) nil))
    (load-onfailure fname))
  (apply dial-name dial-prm-list)
)
;;
;;	load ASISMP.DCL
;;
(defun load-dcl ( / dlgfile)	
  (setq DCL-ID (load_dialog (strcat WORK-DIR DCL-FILE)))	; Load .dcl file
  (while (= DCL-ID -1)
    (while (null dlgfile)
      (setq dlgfile 
        (getfiled "The dialog file is not on the Working Directory. Select asismp.dcl file" WORK-DIR "dcl" 10))
    )
    (setq DCL-ID (load_dialog dlgfile))
  )
)
;;
;; 	Make message dialog box (a la Alert dialog box)
;;
(defun mssg (str)
  (make-dial "msgdlg" 'msg-dialog (list str))
)
;;
;; 	Make question dialog box (a la Alert dialog box)
;;
(defun question (str)
  (eq (make-dial "msgdlg" 'question-dialog (list str)) 'ACCEPT)
)
;;
;; 	Connection report special dialog box 
;;
(defun report-connect (session / conmsglist)
  (setq conmsglist (list (cons "DBMS:" (asi_dbms session)) 
		         (cons "SQL Object Id:" (asi_sql session)) 
		         (cons "Message:" (asi_msg session))))
  (make-dial "conndlg" 'conn-dialog conmsglist)
)
;;
;;	Returns True if there is any connected environment
;;
(defun is-connected ()
  (asi_objp SESSION) 
)
;;
;;	This defun makes connection with environment selected by user from
;;	ENVSDLG dialog box
;;
(defun do-connect (envname username passw )
  ;; connection with environment
  (if (setq SESSION (asi_connect envname username passw))
     (report-connect SESSION)
  )
  SESSION
)
;;
;;	Disconnection from all connected environments
;;
(defun do-disconnect ( / dbmsname)
  (if (asi_objp SESSION)
    (progn  
      (setq dbmsname (asi_dbms SESSION))
      (asi_disconnect SESSION)
      (mssg (strcat MSG_END_OF_SESSION dbmsname)) )
    (mssg MSG_ENV_IS_NOT_CONNECTED)  )
)
;;
;;	MAIN function
;;
(defun c:asismp ( / envnames msg-choice-report retval)
  (maybe-load-asilisp)		; load asilisp.arx if it is absent
  (load-dcl)			; load dialog (asismp.dcl) file
  (setq msg-choice-report "")		; init report string of Choice dialog box

  (while (not (eq retval 'EXIT))	; load CHOICE dialog while not EXIT
     (setq retval (make-dial "choicdlg" 'choice-dialog (list msg-choice-report)))

     (setq msg-choice-report "")		; init report field of Choice dialog box
     (cond 
		;; Connection with environment
	((eq retval 'DO-CONNECT)

		;; make list of environments, described in asi.ini
           (setq envnames (mapcar 'car (asi_getcfg "ENVIRONMENTS")))

		;; If list of connected environments is not empty, Message dialog box:
		;; disconnect from environment first
   	   (if (is-connected)
	      (mssg MSG_DISCONN_ENV_FIRST)
		;; ENVSDLG /select environment dialog/ returns list: '(environm user-name password)
		;; if environment is selected by user,
		;; or 'CANCEL otherwise
     	      (if (listp (setq retval (make-dial "envsdlg" 'envsel-dialog (list envnames))))
	         (if (not (do-connect (car retval) (cadr retval) (caddr retval)) )
  		;; if not connected - message in Report field of Choice dialog box: Press ASIDIAG button
                   (setq msg-choice-report MSG_ERRCONN_PRESS_ASIDIAG_BUTTON) )
	         (setq msg-choice-report MSG_NO_ENV_SELECTED)
	      )))

		;; SQL edit dialog
        ((eq retval 'DO-SQLEDIT)
	   (if (is-connected)
	     (progn
		(setq SESSION (car (asi_objlist)))
     	        (setq retval (make-dial "sqleddlg" 'sqled-dialog nil)) )
             (mssg MSG_CONNECT_ENV_FIRST)  ))

		;; Disconnection from environment
	((eq retval 'DO-DISCONNECT)
	   (do-disconnect) )

		;; Make SQL Diagnostics dialog box
        ((eq retval 'DO-ERRMSG)
	   (make-dial "errmsdlg" 'errmsg-dialog nil))

     ) ; end cond
  )  ; end while
  (princ)
)
             
