;;;
; $Id: cursdlg.lsp,v 1.5 1995/12/20 03:49:54 kirila Exp $
;;;
;;;    cursdlg.lsp
;;;    
;;;    Copyright (C) 1995, 1996 by Autodesk, Inc.
;;;    
;;;    Permission to use, copy, modify, and distribute this software 
;;;    for any purpose and without fee is hereby granted, provided
;;;    that the above copyright notice appears in all copies and 
;;;    that both that copyright notice and the limited warranty and 
;;;    restricted rights notice below appear in all supporting 
;;;    documentation.
;;;    
;;;    AUTODESK PROVIDES THIS PROGRAM "AS IS" AND WITH ALL FAULTS.  
;;;    AUTODESK SPECIFICALLY DISCLAIMS ANY IMPLIED WARRANTY OF 
;;;    MERCHANTABILITY OR FITNESS FOR A PARTICULAR USE.  AUTODESK, INC. 
;;;    DOES NOT WARRANT THAT THE OPERATION OF THE PROGRAM WILL BE 
;;;    UNINTERRUPTED OR ERROR FREE.
;;;    
;;;    Use, duplication, or disclosure by the U.S. Government is subject to 
;;;    restrictions set forth in FAR 52.227-19 (Commercial Computer 
;;;    Software - Restricted Rights) and DFAR 252.227-7013(c)(1)(ii) 
;;;    (Rights in Technical Data and Computer Software), as applicable. 
;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;	This file provide two dialog programs: CURSOR and SINGLE ROW SQL request 
;;;
;;----------------------------------------------------------------------------------
;;	CURSOR DIALOG BOX
;;
(defun cursor-dialog ( / curr-item-number)
   (if (not (new_dialog "cursor_dialog" DCL-ID))	; Initialize the dialog
	(exit))					; exit if this doesn't work

   (action_tile
	"ID_CLOSE_CURS"
	"(close-cursor)(done_dialog)"
   )

   (action_tile
	"ID_UPDATE_CURS"
	"(update-cursor)"
   )

   (action_tile
	"ID_DELETE_CURS"
	"(delete-cursor)"
   )

   (action_tile
	"ID_BUTT_NEXT"
	"(fetch-cursor \"NEXT\")"
   )

   (action_tile
	"ID_BUTT_PRIOR"
	"(fetch-cursor \"PRIOR\")"
   )

   (action_tile
	"ID_BUTT_FIRST"
	"(fetch-cursor \"FIRST\")"
   )

   (action_tile
	"ID_BUTT_LAST"
	"(fetch-cursor \"LAST\")"
   )

   (action_tile
	"ID_SET_CURS"
	"(set-cursor curr-item-number)"
   )

   (setq curr-item-number 0)
   (action_tile
	"ID_LBOX_CURSOR"
	"(write-cursor-name-value (setq curr-item-number (atoi $value)))"
   )

   (action_tile
	"ID_CURS_DIAG"
	"(make-dial \"errmsdlg\" 'errmsg-dialog nil)"
   )

   (init-cursor-dialog) 
   (start_dialog)		; Display the dialogue box
)
;;
;;	Convert any data type to string
;;
(defun tostring (item)
  (cond
    ((eq (type item) 'REAL) 
       (setq item (rtos item)))
    ((eq (type item) 'INT) 
       (setq item (itoa item)))
    ((null item) 
       (setq item "")) )
  item
)

(defun csr-updatedp (csr)
  (= (logand (asi_statecsr csr) 4) 4)
)

(defun set-mode-tyle ()
  (if (= (cursor-type) "SCROLL")
    (progn
      (mode_tile "ID_BUTT_NEXT" 	0)
      (mode_tile "ID_BUTT_PRIOR" 	0)
      (mode_tile "ID_BUTT_FIRST" 	0)
      (mode_tile "ID_BUTT_LAST" 	0)
      (mode_tile "ID_SET_CURS" 		1)
      (mode_tile "ID_UPDATE_CURS" 	1)
      (mode_tile "ID_DELETE_CURS" 	1)
    )
    (progn
      (mode_tile "ID_BUTT_PRIOR" 	1)
      (mode_tile "ID_BUTT_FIRST" 	1)
      (mode_tile "ID_BUTT_LAST" 	1)
      (mode_tile "ID_SET_CURS" 		0)
      (mode_tile "ID_UPDATE_CURS" 	1)
      (mode_tile "ID_DELETE_CURS" 	0)
    )
  )
  (if (not (csr-updatedp CURSOR))
    (progn
      (mode_tile "ID_SET_CURS" 		1)
      (mode_tile "ID_DELETE_CURS" 	1)
    ))  
)

(defun mark-name (name / mark-ch)
  (strcat " " name)
)

(defun unmark-update (name / mark-ch)
  (strcat " " (substr name 2 (- (strlen name) 1)))
)

(defun mark-update (name / mark-ch)
  (setq mark-ch (substr name 1 1))
  (if (/= mark-ch "*")
    (strcat "*" (substr name 2 (- (strlen name) 1))) )
)

(defun updatedp (name)
  (= (substr name 1 1) "*")
)

(defun unmark-list (lst)
  (mapcar '(lambda (item) 
             (if (updatedp (car item))
               (cons (unmark-update (car item)) (cdr item) )
	       item
             )
           ) 
    lst
  )
)

(defun updatedrowp ( / ret)
  (foreach item UPDATELIST 
	(if (updatedp (car item))
	  (setq ret T))
  )
  ret 
)

(defun make-dotlist (h-list c-list)
  (if (null c-list)
    (setq c-list (mapcar '(lambda (e1) "") h-list)) )
  (mapcar '(lambda (e1 e2) (cons (mark-name (cdar e1)) e2)) h-list c-list)
)

(defun set-cursor ( nvalue / name marked-name new_value str-cursor-list)
  (setq name (car (nth nvalue UPDATELIST)))
  (setq marked-name (mark-update name))
  (setq new-value (get_tile "ID_EBOX_UPD"))
  (setq UPDATELIST (subst (cons marked-name new-value) (assoc name UPDATELIST) UPDATELIST))
  (setq str-cursor-list (get-rowstring UPDATELIST))
  (make-cursor-dialog str-cursor-list)
  (if (= (cursor-type) "NON_SCROLL")
    (mode_tile "ID_UPDATE_CURS" 	0)
  )
)

(defun write-cursor-name-value (nvalue / item)	; item = (name.value)
  (setq item (nth nvalue UPDATELIST))
  (set_tile "ID_ARG_NAME" (car item))
  (set_tile "ID_EBOX_UPD" (tostring (cdr item)))
)

(defun make-cursor-dialog (str-cursor-list)
  (start_list "ID_LBOX_CURSOR")
  (mapcar 'add_list str-cursor-list)
  (end_list)
)

(defun update-cursor-dialog (str-cursor-list)
  (start_list "ID_LBOX_CURSOR" 1)
  (mapcar 'add_list str-cursor-list)
  (end_list)
)

(defun get-rowstring (llist)
  (mapcar '(lambda (l) (strcat (car l) "\t" (tostring (cdr l)))) llist)
)

(defun init-cursor-dialog ( / str-cursor-list header-list cursor-list)
  (set-mode-tyle)
  (setq header-list (asi_coldsc CURSOR))	; header of table
  (setq cursor-list (asi_fetch CURSOR))		; first row of table
  (if (null CURSOR-LIST)
    (progn 
      (set_tile "ID_CURSOR_RESULT" MSG_TABL_BOTTOM)
      (mode_tile "ID_SET_CURS" 		1)
      (mode_tile "ID_DELETE_CURS"	1)
    ))
  (setq UPDATELIST (make-dotlist header-list cursor-list))
  (setq str-cursor-list (get-rowstring UPDATELIST))
  (make-cursor-dialog str-cursor-list)
  (write-cursor-name-value 0)
;  (set_tile "ID_TXT_SQLSTM" MSG_SQLSTM_ISEMPTY)
)

(defun update-table ()
  (if (updatedrowp)
    (if (question MSG_QUEST_UPD_TABLE)
      (progn 
        (setq collist (column-list))
        (if (asi_update CURSOR collist) 
          (set_tile "ID_CURSOR_RESULT" MSG_UPD_TABLE)
          (set_tile "ID_CURSOR_RESULT" MSG_ERR_UPD_TABLE)
        )
      )
      (set_tile "ID_CURSOR_RESULT" MSG_ERR_NOTUPD_TABLE)
    )  
    (mode_tile "ID_UPDATE_CURS" 	1)
  )
)

(defun fetch-cursor (orient / header-list cursor-list)
  (set_tile "ID_CURSOR_RESULT" "")
  (update-table)
  (setq header-list (asi_coldsc CURSOR)		; header of table
        cursor-list (asi_fetch CURSOR orient))
  (if (null CURSOR-LIST)
    (progn
      (if (or (= orient "NEXT") (= orient "LAST"))
        (set_tile "ID_CURSOR_RESULT" MSG_TABL_BOTTOM)
        (set_tile "ID_CURSOR_RESULT" MSG_TABL_TOP)
      )
      (mode_tile "ID_SET_CURS" 		1)
      (mode_tile "ID_DELETE_CURS"	1)
    )
  )
  (setq UPDATELIST (make-dotlist header-list cursor-list))
  (setq str-cursor-list (get-rowstring UPDATELIST))
  (update-cursor-dialog str-cursor-list) 
  (write-cursor-name-value 0)
)

(defun column-list ( / header-list collist)
  (setq header-list (asi_coldsc CURSOR))	; header of table
  (mapcar '(lambda (e1 e2) 		; make column list to update
	     (if (updatedp (car e1))
               (setq collist (cons (list (cdar e2) (cdr e1)) collist) ) 
           )) UPDATELIST header-list)
  collist
)

(defun update-cursor ( / collist str-cursor-list)
  (setq collist (column-list))
  (if (asi_update CURSOR collist) 
    (progn 
      (setq UPDATELIST (unmark-list UPDATELIST))
      (setq str-cursor-list (get-rowstring UPDATELIST))
      (update-cursor-dialog str-cursor-list) 
      (write-cursor-name-value 0)
      (mode_tile "ID_UPDATE_CURS" 	1)
      (set_tile "ID_CURSOR_RESULT" MSG_UPDATE_ROW)
    )
    (set_tile "ID_CURSOR_RESULT" MSG_ERR_UPD_TABLE)
  )
)

(defun delete-cursor ()
  (if (question MSG_QUEST_DELETE_ROW)
    (if (asi_delete CURSOR)
      (progn 
        (setq UPDATELIST nil)
        (fetch-cursor "NEXT")
        (set_tile "ID_CURSOR_RESULT" MSG_DELETE_ROW)
        (mode_tile "ID_UPDATE_CURS" 	1)
      )
      (set_tile "ID_CURSOR_RESULT" MSG_ERR_DELETE)
    )
    (set_tile "ID_CURSOR_RESULT" MSG_DELETE_NOT_EXECUTED)
  )
)
;;---------------------------------------------------------------------------------
;;	SINGLEROW DIALOG BOX
;;
(defun singlerow-dialog (rowlist)
   (if (not (new_dialog "single_row_dialog" DCL-ID))	; Initialize the dialog
	(exit))					; exit if this doesn't work

   (action_tile
	"ID_CLOSE_CURS"
	"(done_dialog)"
   )

   (action_tile
	"ID_LBOX_PARS"
	""
   )

   (init-singlerow-dialog rowlist) 
   (start_dialog)		; Display the dialogue box
)

(defun init-singlerow-dialog (row-list / header-list updatelist str-cursor-list)
  (setq header-list (asi_coldsc STATEMENT))	; header of table
  (setq updatelist (make-dotlist header-list row-list))
  (setq str-cursor-list (get-rowstring updatelist))
  (make-cursor-dialog str-cursor-list)
  (write-cursor-name-value 0)
)